@@ -1,4 +1,3 @@
-
 /// Returns this `crate` name.
 ///
 /// # Panics
@@ -430,6 +429,15 @@ macro_rules! app {
         }
     };
 
+    // Option global
+    // clapi::app! { (@option => (global => true/false ) ) }
+    (@option ($option_builder:expr) (alias => $($literal:expr),+) $($tt:tt)*) => {
+        $crate::app!{
+            @option
+            ($option_builder$(.global($literal))+) $($tt)*
+        }
+    };
+
     // Command argument
     // clapi::app! { (@arg value => ( ... ) ) }
     (@command ($builder:expr) (@arg $arg_name:ident $(=> $($rest:tt)+)?) $($tt:tt)*) => {
@@ -615,4 +623,4 @@ macro_rules! run_crate_app {
         .use_default_suggestions()
         .run()
     }};
-}
\ No newline at end of file
+}
